| package |
package := Package name: 'ExternalPipe'.
package paxVersion: 0;
	basicComment: 'Copyright (c) 2004 Udo Schneider

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

22-Jul-2004	Initial Release'.

package basicPackageVersion: '0.002'.


package classNames
	add: #ExternalPipe;
	add: #ExternalPipeExecuteError;
	yourself.

package methodNames
	add: #CRTLibrary -> #convertWinStdHandle:toCRTFileMode:;
	add: #ExternalProcess -> #stderrPipe;
	add: #ExternalProcess -> #stderrPipe:;
	add: #ExternalProcess -> #stdinPipe;
	add: #ExternalProcess -> #stdinPipe:;
	add: #ExternalProcess -> #stdoutPipe;
	add: #ExternalProcess -> #stdoutPipe:;
	add: #KernelLibrary -> #createPipe:hWritePipe:lpPipeAttributes:nSize:;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	add: '..\Burning River\ExternalProcess\ExternalProcess';
	yourself).

package!

"Class Definitions"!

Object subclass: #ExternalPipe
	instanceVariableNames: 'readHandle writeHandle streams'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Error subclass: #ExternalPipeExecuteError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!CRTLibrary methodsFor!

convertWinStdHandle: stdHandle toCRTFileMode: modeString 
	"Private - Convert the standard Win32 OS file handle to a CRT file stream."

	| fd2 hFile |
	fd2 := CRTLibrary default _open_osfhandle: stdHandle flags: 0.
	fd2 < 0 ifTrue: [CRTError signal].
	hFile := CRTLibrary default _fdopen: fd2 mode: modeString.
	^hFile! !
!CRTLibrary categoriesFor: #convertWinStdHandle:toCRTFileMode:!private! !

!ExternalProcess methodsFor!

stderrPipe
	| pipe |
(stderr isKindOf: ExternalPipe) ifFalse: [
	pipe := ExternalPipe new.
	self stderrPipe: pipe.].
	^pipe!

stderrPipe: aPipe 
	stderr := aPipe writeStream.
	si hStdError: aPipe writeHandle!

stdinPipe
	| pipe |
(stdin  isKindOf: ExternalPipe) ifFalse: [

	pipe := ExternalPipe new.
	self stdinPipe: pipe.].
	^pipe!

stdinPipe: aPipe 
	stdin := aPipe readStream.
	si hStdInput: aPipe readHandle!

stdoutPipe
	| pipe |
(stdout  isKindOf: ExternalPipe) ifFalse: [

	pipe := ExternalPipe new.
	self stdoutPipe: pipe.].
	^pipe!

stdoutPipe: aPipe 
	stdout := aPipe writeStream.
	si hStdOutput: aPipe writeHandle! !
!ExternalProcess categoriesFor: #stderrPipe!public! !
!ExternalProcess categoriesFor: #stderrPipe:!private! !
!ExternalProcess categoriesFor: #stdinPipe!public! !
!ExternalProcess categoriesFor: #stdinPipe:!private! !
!ExternalProcess categoriesFor: #stdoutPipe!public! !
!ExternalProcess categoriesFor: #stdoutPipe:!private! !

!KernelLibrary methodsFor!

createPipe: hReadPipe hWritePipe: hWritePipe lpPipeAttributes: lpPipeAttributes nSize: nSize <stdcall: bool CreatePipe handle* handle* lpvoid dword> ^self invalidCall 
! !
!KernelLibrary categoriesFor: #createPipe:hWritePipe:lpPipeAttributes:nSize:!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

ExternalPipe guid: (GUID fromString: '{629C388C-D851-4A03-869A-BC0AA06903A7}')!
ExternalPipe comment: ''!
!ExternalPipe categoriesForClass!Kernel-Objects! !
!ExternalPipe methodsFor!

connectStreams
	streams := Array with: (StdioFileStream 
						fromHandle: (CRTLibrary default convertWinStdHandle: self readHandle toCRTFileMode: 'rt'))
				with: (StdioFileStream 
						fromHandle: (CRTLibrary default convertWinStdHandle: self writeHandle toCRTFileMode: 'wt'))!

createPipe
	| sa bResult |
	readHandle := ExternalHandle new.
	writeHandle := ExternalHandle new.
	sa := SECURITY_ATTRIBUTES new.
	sa bInheritHandle: true asDword.
	KernelLibrary default 
		createPipe: readHandle
		hWritePipe: writeHandle
		lpPipeAttributes: sa
		nSize: 0.
	bResult = false 
		ifTrue: [ExternalPipeExecuteError signal: 'Unable to create Pipe' with: self]!

readHandle
	^readHandle!

readStream
	streams isNil ifTrue: [self connectStreams].
	^streams at: 1!

writeHandle
	^writeHandle!

writeStream
	streams isNil ifTrue: [self connectStreams].
	^streams at: 2! !
!ExternalPipe categoriesFor: #connectStreams!private! !
!ExternalPipe categoriesFor: #createPipe!helpers!initializing!private! !
!ExternalPipe categoriesFor: #readHandle!accessing!private! !
!ExternalPipe categoriesFor: #readStream!accessing!public! !
!ExternalPipe categoriesFor: #writeHandle!accessing!private! !
!ExternalPipe categoriesFor: #writeStream!accessing!public! !

!ExternalPipe class methodsFor!

example1
	"self example1"

	| outputPipe inputPipe process |
	outputPipe := ExternalPipe new.
	inputPipe := ExternalPipe new.
	process := ExternalProcess new.
	process commandLine: 'cmd'.
	process stdoutPipe: outputPipe.
	process stdinPipe: inputPipe.
	[process executeSync] fork.
	
	[Transcript
		show;
		clear.
	[outputPipe readStream atEnd] whileFalse: 
			[Transcript
				nextPut: outputPipe readStream nextLine;
				cr;
				flush]] 
			fork.
	[process isAlive] whileFalse.
	[process isTerminated] whileFalse: 
			[| command |
			command := Prompter prompt: 'Please enter a command...' caption: 'DOS Command'.
			(command notNil and: [command notEmpty]) 
				ifTrue: 
					[(inputPipe writeStream)
						nextPutAll: command;
						cr;
						flush]
				ifFalse: 
					[(inputPipe writeStream)
						nextPutAll: 'exit';
						cr;
						flush]]!

example2
	"self example2"

	| outputPipe inputPipe process |
"Create external proccess"
	process := ExternalProcess new.
	process commandLine: 'cmd'.
"Query Pipes"
	outputPipe := process stdoutPipe.
	inputPipe := process stdinPipe.
"Execute exeternal proccess in different smalltalk process"
	[process executeSync] fork.
	"Start Display to transcript proccess"
	[Transcript
		show;
		clear.
	[outputPipe readStream atEnd] whileFalse: 
			[Transcript
				nextPut: outputPipe readStream nextLine;
				cr;
				flush]] 
			fork.
"Wait untl external process is alive"
	[process isAlive] whileFalse.
"Ask for input as long as the process is running."
	[process isTerminated] whileFalse: 
			[| command |
			command := Prompter prompt: 'Please enter a command...' caption: 'DOS Command'.
			(command notNil and: [command notEmpty]) 
				ifTrue: 
					[(inputPipe writeStream)
						nextPutAll: command;
						cr;
						flush]
				ifFalse: 
					[(inputPipe writeStream)
						nextPutAll: 'exit';
						cr;
						flush]]!

new
	^super new createPipe; yourself! !
!ExternalPipe class categoriesFor: #example1!public! !
!ExternalPipe class categoriesFor: #example2!public! !
!ExternalPipe class categoriesFor: #new!instance creation!public! !

ExternalPipeExecuteError guid: (GUID fromString: '{9369F27F-9315-465D-B0DB-A258C5D4047A}')!
ExternalPipeExecuteError comment: ''!
!ExternalPipeExecuteError categoriesForClass!Kernel-Exception Handling! !
"Binary Globals"!

"Resources"!

