| package |
package := Package name: 'US Treemap - Render Abstract'.
package paxVersion: 0;
	basicComment: ''.

package basicPackageVersion: '0.019'.


package classNames
	add: #AbstractTreeMapRenderStrategy;
	add: #LightningColorCache;
	add: #LightningRenderStrategy;
	add: #TreeMapRenderState;
	yourself.

package methodNames
	add: #Color -> #darkenBy:;
	add: 'Pen class' -> #null;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\..\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	yourself).

package!

"Class Definitions"!

Object subclass: #AbstractTreeMapRenderStrategy
	instanceVariableNames: 'renderer bitmap canvas'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #LightningColorCache
	instanceVariableNames: 'cache fillBlock'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #TreeMapRenderState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
AbstractTreeMapRenderStrategy subclass: #LightningRenderStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!Color methodsFor!

darkenBy: aNumber 
	"Return the color darkened by aNumber (0-255).
		0: Don't darken at all
		255: Maximum darkening (Black)"

	| factor rgb |
	factor := (255 - aNumber) / 255.
	rgb := self asRGB.
	^Color 
		red: rgb red * factor
		green: rgb green * factor
		blue: rgb blue * factor! !
!Color categoriesFor: #darkenBy:!converting!public! !

!Pen class methodsFor!

null


	^self fromLogPen: ((LOGPEN new)
				style: PS_NULL;
			
				yourself)! !
!Pen class categoriesFor: #null!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

AbstractTreeMapRenderStrategy guid: (GUID fromString: '{DE4ED926-D19A-428A-A19B-4FB1B6AAB08B}')!
AbstractTreeMapRenderStrategy comment: ''!
!AbstractTreeMapRenderStrategy categoriesForClass!Unclassified! !
!AbstractTreeMapRenderStrategy methodsFor!

initialize!

initialRenderState
	^TreeMapRenderState new!

render: node using: renderState 
	self subclassResponsibility!

renderNode: node using: renderState 
	node isLeafNode ifTrue: [self render: node using: renderState]!

setBitmap: anObject 
	bitmap := anObject.
canvas := bitmap canvas.!

setCanvas: anObject 
bitmap := nil.
	canvas := anObject!

setRenderer: aTreeMapRenderer
	renderer := aTreeMapRenderer! !
!AbstractTreeMapRenderStrategy categoriesFor: #initialize!private! !
!AbstractTreeMapRenderStrategy categoriesFor: #initialRenderState!public! !
!AbstractTreeMapRenderStrategy categoriesFor: #render:using:!public! !
!AbstractTreeMapRenderStrategy categoriesFor: #renderNode:using:!public! !
!AbstractTreeMapRenderStrategy categoriesFor: #setBitmap:!accessing!private! !
!AbstractTreeMapRenderStrategy categoriesFor: #setCanvas:!accessing!private! !
!AbstractTreeMapRenderStrategy categoriesFor: #setRenderer:!accessing!public! !

AbstractTreeMapRenderStrategy methodProtocol: #'Treemap Render Strategy' attributes: #(#readOnly) selectors: #(#initialRenderState #render:using: #renderNode:using: #setBitmap: #setCanvas: #setRenderer:)!

!AbstractTreeMapRenderStrategy class methodsFor!

description
^self class displayString!

new
^super new initialize! !
!AbstractTreeMapRenderStrategy class categoriesFor: #description!public! !
!AbstractTreeMapRenderStrategy class categoriesFor: #new!public! !

LightningColorCache guid: (GUID fromString: '{7702B83E-C089-4788-90B8-1EC08A5B6FAE}')!
LightningColorCache comment: ''!
!LightningColorCache categoriesForClass!Kernel-Objects! !
!LightningColorCache methodsFor!

at: aColor 
	^cache at: aColor ifAbsentPut: [ (fillBlock value: aColor)]!

setCacheSize: anInteger 
	cache := LookupTable new: anInteger!

setFillBlock: aBlock 
	fillBlock := aBlock! !
!LightningColorCache categoriesFor: #at:!private! !
!LightningColorCache categoriesFor: #setCacheSize:!private! !
!LightningColorCache categoriesFor: #setFillBlock:!private! !

!LightningColorCache class methodsFor!

cacheSize: anInteger fillBlock: aBlock 
	^(self basicNew)
		setCacheSize: anInteger;
		setFillBlock: aBlock;
		yourself!

defaultCacheSize
	^32!

fillBlock: aBlock 
^self cacheSize: self defaultCacheSize  fillBlock: aBlock !

new
	self shouldNotImplement! !
!LightningColorCache class categoriesFor: #cacheSize:fillBlock:!public! !
!LightningColorCache class categoriesFor: #defaultCacheSize!public! !
!LightningColorCache class categoriesFor: #fillBlock:!public! !
!LightningColorCache class categoriesFor: #new!public! !

TreeMapRenderState guid: (GUID fromString: '{86C9488C-CF7D-4416-8EEB-0D4EF4F9E763}')!
TreeMapRenderState comment: ''!
!TreeMapRenderState categoriesForClass!Unclassified! !
!TreeMapRenderState methodsFor!

setRootRectangle: aRectangle 
	!

update: aNode horizontalIn: aRectangle	^self!

update: aNode in: aRectangle 
	^self!

update: aNode verticalIn: aRectangle 	^self! !
!TreeMapRenderState categoriesFor: #setRootRectangle:!public! !
!TreeMapRenderState categoriesFor: #update:horizontalIn:!public! !
!TreeMapRenderState categoriesFor: #update:in:!public! !
!TreeMapRenderState categoriesFor: #update:verticalIn:!public! !

LightningRenderStrategy guid: (GUID fromString: '{3C41E114-2BCB-4255-AAF3-C3AF7BD0988A}')!
LightningRenderStrategy comment: ''!
!LightningRenderStrategy categoriesForClass!Unclassified! !
!LightningRenderStrategy methodsFor!

ambientLightIntensity
	^55!

directLightIntensity
	^200!

renderStep
	^4! !
!LightningRenderStrategy categoriesFor: #ambientLightIntensity!public! !
!LightningRenderStrategy categoriesFor: #directLightIntensity!public! !
!LightningRenderStrategy categoriesFor: #renderStep!public! !

"Binary Globals"!

"Resources"!

