| package |
package := Package name: 'US Treemap - Base'.
package paxVersion: 0;
	basicComment: 'path := ''S:\lobtest''.
complexTree := VirtualTreeModel withRoots: (Array with: (BonsaiFolder fromString: path)).
complexTree hasChildrenBlock: [:each | each children notEmpty].
complexTree getChildrenBlock: [:each | each children].
dib := DIBSection 
			width: 640
			height: 480
			depth: 32.
renderer := TreeMapRenderer new.
renderer
	renderStrategy: CircleRenderStrategy new;
	setBitmap: dib;
	setLayoutRectangle: (0 @ 0 extent: dib extent);
	setTreeModel: complexTree;
	render.
ImagePresenter showOn: dib
'.

package basicPackageVersion: '0.022'.


package classNames
	add: #TreeMapModel;
	add: #TreeMapNode;
	add: #TreeMapRenderer;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\Object Arts\Dolphin\MVP\Models\Tree\Dolphin Tree Models';
	add: 'Layout\US Treemap - Layout Slice and Dice';
	add: 'Render\US Treemap - Render Flat';
	yourself).

package!

"Class Definitions"!

Object subclass: #TreeMapModel
	instanceVariableNames: 'treeModel rootNode renderer'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #TreeMapNode
	instanceVariableNames: 'object parent model children size layoutRectangle'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Object subclass: #TreeMapRenderer
	instanceVariableNames: 'treeMapModel layoutRectangle nodeSizeBlock nodeColorBlock layoutStrategy renderStrategy'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

TreeMapModel guid: (GUID fromString: '{85BE892B-E1E9-42D3-9F34-CD2BFE1AC8AA}')!
TreeMapModel comment: ''!
!TreeMapModel categoriesForClass!Unclassified! !
!TreeMapModel methodsFor!

childNodeAt: aPoint 
	^rootNode childNodeAt: aPoint!

initialize
	rootNode := TreeMapNode 
				object: nil
				parent: nil
				model: self.
	!

leafNodeAt: aPoint 
	^rootNode leafNodeAt: aPoint!

nodeForObject: anObject 
	| parents node |
	parents := (treeModel allParentsOf: anObject) copyWith: anObject.
	node := self rootNode.
	parents 
		do: [:eachParent | node := node children detect: [:eachNode | eachNode object = eachParent]].
	^node!

populateNode: node 
	node children do: 
			[:eachNode | 
			eachNode 
				children: ((treeModel childrenOf: eachNode object) collect: 
							[:eachObject | 
							TreeMapNode 
								object: eachObject
								parent: node
								model: self]).
			self populateNode: eachNode]!

populateRootNode
	rootNode := TreeMapNode 
				object: nil
				parent: nil
				model: self.
	rootNode 
		children: (treeModel roots collect: 
					[:eachRoot | 
					TreeMapNode 
						object: eachRoot
						parent: rootNode
						model: self]).
	self populateNode: rootNode!

rootNode
	^rootNode!

setModel: aTreeOrListModel 
	treeModel := (aTreeOrListModel isKindOf: TreeModelAbstract) 
				ifTrue: [aTreeOrListModel]
				ifFalse: [TreeModel withRoots: aTreeOrListModel  list ].
	self populateRootNode!

setRenderer: aTreeMapRenderer
renderer := aTreeMapRenderer!

sizeOf: anObject 
	^renderer nodeSize:  anObject!

treeModel
	^treeModel! !
!TreeMapModel categoriesFor: #childNodeAt:!public! !
!TreeMapModel categoriesFor: #initialize!private! !
!TreeMapModel categoriesFor: #leafNodeAt:!public! !
!TreeMapModel categoriesFor: #nodeForObject:!public! !
!TreeMapModel categoriesFor: #populateNode:!private! !
!TreeMapModel categoriesFor: #populateRootNode!private! !
!TreeMapModel categoriesFor: #rootNode!accessing!public! !
!TreeMapModel categoriesFor: #setModel:!private! !
!TreeMapModel categoriesFor: #setRenderer:!private! !
!TreeMapModel categoriesFor: #sizeOf:!public! !
!TreeMapModel categoriesFor: #treeModel!public! !

!TreeMapModel class methodsFor!

new
		^self shouldNotImplement!

on: aTreeModel renderer: aTreeMapRenderer 
	^(self basicNew)
		initialize;
		setRenderer: aTreeMapRenderer;
		setModel: aTreeModel;
		yourself! !
!TreeMapModel class categoriesFor: #new!public! !
!TreeMapModel class categoriesFor: #on:renderer:!public! !

TreeMapNode guid: (GUID fromString: '{69FA6A9B-602D-480B-A153-9056740802CA}')!
TreeMapNode comment: ''!
!TreeMapNode categoriesForClass!Unclassified! !
!TreeMapNode methodsFor!

childNodeAt: aPoint 
	^self children detect: [ :each | (each layoutRectangle containsPoint: aPoint)] ifNone: [nil].
"
	self children do: [:each | (each layoutRectangle containsPoint: aPoint) ifTrue: [^each]].
	^nil"!

children
	^children !

children: anArray 
	children := anArray!

childrenWithExtent
	^children select: [:each | each hasExtent]!

hasExtent
	^self size strictlyPositive!

hasParent
	^parent notNil!

isLeafNode
	^children isEmpty!

layoutRectangle
	^layoutRectangle!

leafNodeAt: aPoint 
	"Can use detect or #childNodeAt: here?"

	#USTodo.
	self children do: 
			[:each | 
			(each layoutRectangle containsPoint: aPoint) 
				ifTrue: [each isLeafNode ifFalse: [^each leafNodeAt: aPoint] ifTrue: [^each]]].
	^nil!

object
	"Private - Answer the <Object> which is the element stored in this node."

	^object!

parent
	"Private - Answer the <MoenTreeNode> which is the receiver's parent node."

	^parent!

parent: aMoenTreeNode
	parent := aMoenTreeNode!

printOn: aStream 
	super printOn: aStream.
	aStream
		nextPutAll: ' (';
		print:		object;
		nextPutAll: ')'!

setLayoutRectangle: aRectangle 
	
	layoutRectangle := aRectangle !

setObject: anObject parent: aTreeMapNode model:  aTreeMapModel 
	object := anObject.
	parent := aTreeMapNode.
model :=  aTreeMapModel .
	layoutRectangle := Rectangle new.
	^self!

size
	"Use #inject:into: here?"

	#USTodo.
	size isNil 
		ifTrue: 
			[size := self isLeafNode 
						ifTrue: [model sizeOf: object]
						ifFalse: 
							[self children inject: 0 into: [ :sum :each | sum + each size]]].
	^size! !
!TreeMapNode categoriesFor: #childNodeAt:!public! !
!TreeMapNode categoriesFor: #children!enumerating!public! !
!TreeMapNode categoriesFor: #children:!accessing!private! !
!TreeMapNode categoriesFor: #childrenWithExtent!enumerating!public! !
!TreeMapNode categoriesFor: #hasExtent!public! !
!TreeMapNode categoriesFor: #hasParent!public! !
!TreeMapNode categoriesFor: #isLeafNode!public! !
!TreeMapNode categoriesFor: #layoutRectangle!accessing!public! !
!TreeMapNode categoriesFor: #leafNodeAt:!public! !
!TreeMapNode categoriesFor: #object!accessing!public! !
!TreeMapNode categoriesFor: #parent!accessing!public! !
!TreeMapNode categoriesFor: #parent:!accessing!public! !
!TreeMapNode categoriesFor: #printOn:!public! !
!TreeMapNode categoriesFor: #setLayoutRectangle:!accessing!private! !
!TreeMapNode categoriesFor: #setObject:parent:model:!initializing!private! !
!TreeMapNode categoriesFor: #size!public! !

!TreeMapNode class methodsFor!

new
	"Use #object:parent: to create nodes"

	^self shouldNotImplement!

object: anObject parent: aMoenTreeNode model: aTreeMapModel 
	^(self basicNew)
		initialize;
		setObject: anObject
			parent: aMoenTreeNode
			model: aTreeMapModel;yourself! !
!TreeMapNode class categoriesFor: #new!public! !
!TreeMapNode class categoriesFor: #object:parent:model:!instance creation!public! !

TreeMapRenderer guid: (GUID fromString: '{CF4B34C3-5F8B-4152-9413-4A9E5F1844DE}')!
TreeMapRenderer comment: ''!
!TreeMapRenderer categoriesForClass!Unclassified! !
!TreeMapRenderer methodsFor!

defaultNodeColorBlock ^ Compiler evaluate: '[ :object |  Color  white]'.!

defaultNodeSizeBlock ^Compiler evaluate: '[ :object | object size]'.!

initialize

	nodeColorBlock := self defaultNodeColorBlock.
	nodeSizeBlock := self defaultNodeSizeBlock.
	layoutStrategy := SliceAndDiceLayoutStrategy new.
	renderStrategy := FlatRenderStrategy new!

layoutStrategy
	^layoutStrategy!

layoutStrategy: anObject 
	layoutStrategy := anObject.self trigger: #renderSettingsChanged!

nodeColor: anObject 
	^nodeColorBlock value: anObject!

nodeColorBlock
	^nodeColorBlock!

nodeColorBlock: anObject 
	nodeColorBlock := anObject.
	self trigger: #renderSettingsChanged!

nodeSize: anObject 
	^nodeSizeBlock value: anObject.
	!

nodeSizeBlock
	^nodeSizeBlock!

nodeSizeBlock: anObject 
	nodeSizeBlock := anObject .self trigger: #renderSettingsChanged!

render
	| renderState |
	renderStrategy setRenderer: self.
	renderState := (renderStrategy initialRenderState)
				setRootRectangle: layoutRectangle;
				yourself.
	layoutStrategy setRenderStrategy: renderStrategy.
	layoutStrategy 
		layout: treeMapModel
		in: layoutRectangle
		using: renderState!

renderStrategy
	^renderStrategy!

renderStrategy: anObject 
	renderStrategy := anObject.self trigger: #renderSettingsChanged!

setBitmap: aBitmap 
renderStrategy
		setBitmap: aBitmap !

setCanvas: aCanvas 
	renderStrategy setCanvas: aCanvas !

setLayoutRectangle: aRectangle 
	layoutRectangle := aRectangle!

setModel: aTreeOrListModel 
	treeMapModel := TreeMapModel on: aTreeOrListModel  renderer: self!

treeMapModel
	^treeMapModel! !
!TreeMapRenderer categoriesFor: #defaultNodeColorBlock!public! !
!TreeMapRenderer categoriesFor: #defaultNodeSizeBlock!public! !
!TreeMapRenderer categoriesFor: #initialize!public! !
!TreeMapRenderer categoriesFor: #layoutStrategy!accessing!public! !
!TreeMapRenderer categoriesFor: #layoutStrategy:!accessing!public! !
!TreeMapRenderer categoriesFor: #nodeColor:!accessing!public! !
!TreeMapRenderer categoriesFor: #nodeColorBlock!accessing!public! !
!TreeMapRenderer categoriesFor: #nodeColorBlock:!accessing!public! !
!TreeMapRenderer categoriesFor: #nodeSize:!accessing!public! !
!TreeMapRenderer categoriesFor: #nodeSizeBlock!accessing!public! !
!TreeMapRenderer categoriesFor: #nodeSizeBlock:!accessing!public! !
!TreeMapRenderer categoriesFor: #render!public! !
!TreeMapRenderer categoriesFor: #renderStrategy!accessing!public! !
!TreeMapRenderer categoriesFor: #renderStrategy:!accessing!public! !
!TreeMapRenderer categoriesFor: #setBitmap:!public! !
!TreeMapRenderer categoriesFor: #setCanvas:!public! !
!TreeMapRenderer categoriesFor: #setLayoutRectangle:!public! !
!TreeMapRenderer categoriesFor: #setModel:!public! !
!TreeMapRenderer categoriesFor: #treeMapModel!public! !

!TreeMapRenderer class methodsFor!

availableLayoutStrategies
	^(ClassCategory name: 'Treemap-Layout') contents!

availableRenderStrategies
	^(ClassCategory name: 'Treemap-Render') contents !

new
^super new initialize! !
!TreeMapRenderer class categoriesFor: #availableLayoutStrategies!public! !
!TreeMapRenderer class categoriesFor: #availableRenderStrategies!public! !
!TreeMapRenderer class categoriesFor: #new!public! !

"Binary Globals"!

"Resources"!

